!
!	test.f90  by Ian Ollmann
!	
!	Copyright (c) 2003 Apple Computer. All rights reserved.
!
!	This sample code illustrates how to call vector code from within
!	a FORTRAN program. We compare several different methods of 
!	calculating exp(x). We start with the regular libm method, which
!	operates on the data one at a time. Next we move to a C wrapper
!	function vexpf_wrapper that reexports the vecLib vexpf() function
!	with FORTRAN calling APIs. Finally there is a function that uses
!	the AltiVec instruction for estimating 2**x to calculate an estimate
!	of exp(x). It is written entirely in C. 
!
!	Before compiling this file, you must compile its two companions in C:
!
!		gcc -c -O3 -funroll-loops -faltivec vector.c
!		gcc -c -O3 timer.c
!
!	Then compile this code to use those object files. This example
!	was prepared using the MacOS X xlf beta Compiler arguments may vary 
!	by compiler:
!
!		f90 vector.o timer.o test.f90 -Wl,-framework -Wl,vecLib -O3 -o test
!
!	To run:
!
!		./test
!


double precision time, besttime, overhead
real*4 in(4096), out(4096)
integer i, j


! Calculate the amount of overhead involved in calling the fast PPC clock
! startclock and stopclock are implemented in timer.o. 
! startclock is not thread safe
overhead = 1e20
do j = 1,1000
	call startclock()
	call stopclock( time )
	if( time < overhead )  overhead = time 
enddo
write(6,*)
write (6,*) 'clock overhead: ', overhead * 1000000, 'microseconds.'
write(6,*)
write(6,*)  'Benchmarking....'
write(6,*)

!move the data into the caches
do i=1,4096
	in(i) = 1.
	out(i) = 1.
enddo

! We run our timing loop here several times and take the best time
! We do this because there is always the potential that the OS will preemptively
! swap us out for another process, interrupting our timing result. The 
! best time out of 1000 is very likely one in which we didn't lose the processor
! This makes the measurement more consistent. It also removes effects like cache
! misses from causing trouble. This is a better assay for code quality. When
! tuning cache behavior, obviously you want to look at larger data sets and 
! longer periods of time. 

!measure time for 4096 calls to exp()
besttime = 1e20
do j = 1,10000
	call startclock()
	do i=1,4096
		out(i) = exp( in(i))
	enddo
	call stopclock( time )
	if( time < bestTime )  besttime = time 
enddo
write(6,*) 'Time to call exp() 4096 times: ', (time - overhead) * 1000000., 'microseconds.'
write(6,*)

!measure time to process 4096 floats with the vecLib vexpf function
besttime = 1e20
do j = 1,10000
	call startclock()
	call vexpf_wrapper( in, out, 4096 )
	call stopclock( time )
	if( time < bestTime )  besttime = time 
enddo
write(6,*) 'Time for vexpf to repeat this calculation: ', (time - overhead) * 1000000., 'microseconds.'
write(6,*)

!measure time to process 4096 floats with the Altivec exp estimate
besttime = 1e20
do j = 1,10000
	call startclock()
	call vexp_estimate( in, out, 4096 )
	call stopclock( time )
	if( time < bestTime )  besttime = time 
enddo
write(6,*) 'Time for vexp_estimate to estimate exp() over that dataset: ', (time - overhead) * 1000000., 'microseconds.'
write(6,*)


end
