//
//	Sample code by Ian Ollmann 
//	
//	Copyright (c) Apple Computer 2003. All Rights Reserved
//
//	These are some simple functions showing various ways
//	to calculate the exponential of an array of floats.
//	
//	vexp_estimate uses the AltiVec exponential estimate
//	instruction. 
//
//	Note that we haven't gone through any effort to optimize 
//	these functions beyond simple vectorization. Loop unrolling, 
//	prefetching and other common methods should make it quite a 
//	bit faster. 
//	
//	These functions assume that the arrays that they operate are
//	aligned to 16 byte bounds. 
//
//	To compile:
//
//		gcc -c -O3 -funroll-loops -faltivec vector.c
//
//	You may also try:
//
//		gcc -c -O3 -fast -funroll-loops -mcpu=G4 -faltivec vector.c
//

#include <vecLib/vfp.h>


//
// This calls the more accurate and slower vexpf() in vecLib.
//
void vexpf_wrapper( vector float *in, vector float *out, int *n )
{
	int i; 
        int loopCount = *n / vec_step( vector float );
        
        //This isn't really optimized as written.
        //-funroll-loops will probably help
        for( i = 0; i < loopCount; i++ )                          
                out[i] = vexpf( in[i] );

}

//Because the vector exp estimate estimates 2**x not e**x, we have to do
//a bit of extra work here:
//
// For:
//	e**x = 2**a
//		
// solve for a
//	x = a*ln(2)
//	x * 1/ln(2) = a
//
// back sub to find e**x expressed as 2**f(x)
//
//	e**x = 2**a = 2**(x*1/ln(2)) = 2**(x*c)		c = 1.0/ln(2.0)
//
// vec_expte is only accurate to one part in 32 error (about +-3%)
void vexp_estimate( vector float *in, vector float *out, int *n )
{
	int i;
	int loopCount = *n / vec_step( vector float );
        vector float oneOverLn2 = (vector float)( 1.442695041 ); //1.0/ln(2)
        vector float zero = (vector float) vec_splat_u32(0);
    
	//This isn't really optimized as written.
	//-funroll-loops will probably help
	for( i = 0; i < loopCount; i++ )
		out[i] = vec_expte( vec_madd( in[i], oneOverLn2, zero) ); 
}

